/* This file was generated by upb_generator from the input file:
 *
 *     envoy/service/discovery/v3/discovery.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef ENVOY_SERVICE_DISCOVERY_V3_DISCOVERY_PROTO_UPB_H__UPB_MINITABLE_H_
#define ENVOY_SERVICE_DISCOVERY_V3_DISCOVERY_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__service__discovery__v3__ResourceLocator_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__ResourceLocator_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__ResourceLocator__DynamicParametersEntry_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__ResourceLocator__DynamicParametersEntry_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__ResourceName_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__ResourceName_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__DiscoveryRequest_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__DiscoveryRequest_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__DiscoveryResponse_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__DiscoveryResponse_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__DeltaDiscoveryRequest_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__DeltaDiscoveryRequest_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__DeltaDiscoveryRequest__InitialResourceVersionsEntry_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__DeltaDiscoveryRequest__InitialResourceVersionsEntry_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__DeltaDiscoveryResponse_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__DeltaDiscoveryResponse_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__DynamicParameterConstraints_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__DynamicParameterConstraints_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint__Exists_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint__Exists_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__DynamicParameterConstraints__ConstraintList_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__DynamicParameterConstraints__ConstraintList_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__Resource_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__Resource_msg_init_ptr;
extern const upb_MiniTable envoy__service__discovery__v3__Resource__CacheControl_msg_init;
extern const upb_MiniTable* envoy__service__discovery__v3__Resource__CacheControl_msg_init_ptr;

extern const upb_MiniTableFile envoy_service_discovery_v3_discovery_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_SERVICE_DISCOVERY_V3_DISCOVERY_PROTO_UPB_H__UPB_MINITABLE_H_ */
